/*
 ********** Begin Standard Header - Do not add comments here ***********
 * 
 * DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 * sample code created by IBM Corporation.  This sample code is
 * not part of any standard or IBM product and is provided to you
 * solely for the purpose of assisting you in the development of
 * your applications.  The code is provided "AS IS", without
 * warranty of any kind.  IBM shall not be liable for any damages
 * arising out of your use of the sample code, even if they have
 * been advised of the possibility of such damages.
 * 
 ************************** End Standard Header ***********************/

import java.util.*;
import com.tivoli.dms.api.*;
import com.tivoli.dms.api.proxy.*;
import java.net.URL;
public class SoftwareDistJobSample implements ClientAPIConstants {
    //public static final String copyright = com.tivoli.dms.common.Copyright.COPYRIGHT;

    //This sample shows how to construct a Software object and then use it in a
    //a Software distribution job to a Wince device with the DEVICE_NAME of "wince1".
    //It also shows two possible ways of building a software removal job to remove the package
    //from the device.
    //Notes: 
    //- The Wince device must already exist in the database.
    //- The sample assumes that the software type of "WINCE_PACKAGE" has been registered.
    //- It assume that there is a Wince software package at http://localhost/snake/snakePackage.txt 
    //  on the DM Server web server.
    //- For the software removal job, it assumes there is a software removal package created in the
    //  database with the name "Snake Package removal".

    //It also shows how to implement ConstantsForAPIs so the constants can
    //be used in the code.

    // You may need to change the URL hostname and port of the DMS Server in the setEndPoint method
    // to match your environment.

    public static void main(String[] argList) {                 
        try {
           String device_name = "wince1";
            //Create Proxies
            JobManagerProxy proxy = new JobManagerProxy();
            proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/JobManagerService"));
            DeviceManagerProxy device_proxy = new DeviceManagerProxy();
            device_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/DeviceManagerService"));
            SoftwareManagerProxy sw_proxy = new SoftwareManagerProxy();
            sw_proxy.setEndPoint(new URL("http://localhost:80/dmserver/services/SoftwareManagerService"));

            //Get device id from name
            Device device = device_proxy.getDeviceFromName(device_name, "Wince");

            //create Software object and specify URL to software package
            Software sw = new Software();
            sw.setSoftwareType("WINCE_PACKAGE");
            sw.setSoftwareURL("http://localhost/snake/snakepackage.txt");
            //sw.setSoftwareName("testsw");
            //sw.setSoftwareVersion("v1");
            //sw.setSoftwareDescription("sw desc");

            //create Software on DM Server
            long swid = sw_proxy.createSoftware(sw, null, null);
            System.out.println("Software created, id = " + swid);

            //get Software from DM Server and print sw name and version from package
            sw = sw_proxy.getSoftware(swid);
            System.out.println("Software name = " + sw.getSoftwareName());
            System.out.println("Software version = " + sw.getSoftwareVersion());

            //Setup job. 
            Job job1 = new Job();
            job1.setDeviceClassName("Wince");
            job1.setJobType(DMS__SOFTWARE_DISTRIBUTION_JOB_TYPE);
            job1.setJobPriority(3);
            job1.setJobDescription("my new sw dist job");
            
            //Set job parm - id of sw package
            HashMap cfg_parms = new HashMap();
            cfg_parms.put(DMS__ID_OF_SOFTWARE_PACKAGE, Long.toString(swid));   
            job1.setJobParmsMap(cfg_parms);

            //Set target device
            long target_devices[] ={device.getDeviceID()};
            job1.setDevicesForJob(target_devices);

            //Create the job and return the job_id
            long job_id = proxy.createJob(job1);
            System.out.println("\nSoftware Distribution job created for device " + device_name + " , id = " + job_id);

            //----------------------------------------------------------------------
            //Now we want to create a software removal job for the snakePackage
            //Here we assume the install package and remove package are the same. 
            //----------------------------------------------------------------------
            job1 = new Job();
            job1.setDeviceClassName("Wince");
            job1.setJobType(DMS__SOFTWARE_REMOVAL_JOB_TYPE);
            job1.setJobPriority(3);
            job1.setJobDescription("remove snake pkg");
            
            //Set job parm - use sw_id of install package which is also used to remove the package
            //from the device
            cfg_parms = new HashMap();
            cfg_parms.put(DMS__ID_OF_SOFTWARE_PACKAGE, Long.toString(swid));   
            job1.setJobParmsMap(cfg_parms);

            //Set target device
            //target_devices[] ={device.getDeviceID()};
            job1.setDevicesForJob(target_devices);

            //Create the job and return the job_id
            job_id = proxy.createJob(job1);
            System.out.println("\nSoftware Removal job created for device " + device_name + " , id = " + job_id);

            //--------------------------------------------------------------------------------
            //The install package and remove packages could also be different.
            //Assume we already have a sw removal package called "Snake Package removal".
            //We specify an additional job parm that contains the sw_id of the package
            //being removed. This results in it getting removed from the INSTALLED_SW table
            //when the job completes.
            //--------------------------------------------------------------------------------

            //First get sw_id of removal package
            Query query = new Query();
            QueryClause qc = new QueryClause();
            qc.setAttribute(SOFTWARE_NAME);
            qc.setOperator(EQUAL);
            qc.setValue("Snake Package removal");
            query.setQueryClause(qc);
            Software[] sws = sw_proxy.getSoftwareFromQuery(query);
            if (sws==null || sws.length==0) {
               System.out.println("Software removal package not found - " + "Snake Package removal");
               return;
            }
            Software remove_pkg = sws[0];

            //Now set up sw removal job
            job1 = new Job();
            job1.setDeviceClassName("Wince");
            job1.setJobType(DMS__SOFTWARE_REMOVAL_JOB_TYPE);
            job1.setJobPriority(3);
            job1.setJobDescription("remove snake pkg 2");
            
            //Set job parm - use sw_id of removal package which we just retrieved
            //from the device.
            cfg_parms = new HashMap();
            cfg_parms.put(DMS__ID_OF_SOFTWARE_PACKAGE, Long.toString(remove_pkg.getSoftwareID()));   

            //Also store sw_id of installed package so that it gets removed from the INSTALLED_SW table
            //when the job completes.
            cfg_parms.put(DMS__ID_OF_SOFTWARE_PACKAGE_TO_REMOVE, Long.toString(swid));   
            job1.setJobParmsMap(cfg_parms);

            //Set target device
            //target_devices[] ={device.getDeviceID()};
            job1.setDevicesForJob(target_devices);

            //Create the job and return the job_id
            job_id = proxy.createJob(job1);
            System.out.println("\nSoftware Removal job created for device " + device_name + " , id = " + job_id);

            return;
        }catch (java.net.MalformedURLException ue) {
           System.out.println("Exception in api - " + ue);
        }catch (DeviceManagerException se) {
            System.out.println("Exception in api - " + se);
        }
    }
}
